@@ -2,15 +2,28 @@ from copy import deepcopy
 from datetime import date, datetime
 from enum import Enum
 import json
+import os
+import re
 import sys
 from typing import Any, Dict, List, Optional, Type, Union
 
-from pydantic import UUID4, BaseModel, ByteSize, EmailStr, Field, HttpUrl, validator
+from pydantic import (
+    UUID4,
+    AnyHttpUrl,
+    BaseModel,
+    ByteSize,
+    EmailStr,
+    Field,
+    HttpUrl,
+    root_validator,
+    validator,
+)
 from pydantic.main import ModelMetaclass
 
 from .consts import DANDI_SCHEMA_VERSION
 from .model_types import (
     AccessTypeDict,
+    AgeReferenceTypeDict,
     DigestTypeDict,
     IdentifierTypeDict,
     LicenseTypeDict,
@@ -25,8 +38,20 @@ if sys.version_info < (3, 8):
 else:
     from typing import Literal
 
+# Local or test deployments of dandi-api will insert URLs into the schema that refer to the domain
+# localhost, which is not a valid TLD. To make the metadata valid in those contexts, setting this
+# environment variable will use a less restrictive pydantic field that allows localhost.
+if "DANDI_ALLOW_LOCALHOST_URLS" in os.environ:
+    HttpUrl = AnyHttpUrl  # noqa: F811
+
 
 NAME_PATTERN = r"^([\w\s\-]+)?,\s+([\w\s\-\.]+)?$"
+UUID_PATTERN = (
+    "[a-f0-9]{8}[-]*[a-f0-9]{4}[-]*" "[a-f0-9]{4}[-]*[a-f0-9]{4}[-]*[a-f0-9]{12}$"
+)
+ASSET_UUID_PATTERN = r"^dandiasset:" + UUID_PATTERN
+DANDI_DOI_PATTERN = r"^10.(48324|80507)/dandi\.\d{6}/\d+\.\d+\.\d+"
+DANDI_PUBID_PATTERN = r"^DANDI:\d{6}/\d+\.\d+\.\d+"
 
 
 def create_enum(data):
@@ -68,6 +93,7 @@ if len(AccessTypeDict["@graph"]) > 2:
     AccessTypeDict["@graph"].pop()
     AccessTypeDict["@graph"].pop()
 AccessType = create_enum(AccessTypeDict)
+AgeReferenceType = create_enum(AgeReferenceTypeDict)
 RoleType = create_enum(RoleTypeDict)
 RelationType = create_enum(RelationTypeDict)
 ParticipantRelationType = create_enum(ParticipantRelationTypeDict)
@@ -109,6 +135,27 @@ class DandiBaseModelMetaclass(ModelMetaclass):
 class DandiBaseModel(BaseModel, metaclass=DandiBaseModelMetaclass):
     id: Optional[str] = Field(description="Uniform resource identifier", readOnly=True)
 
+    def json_dict(self):
+        """
+        Recursively convert the instance to a `dict` of JSONable values,
+        including converting enum values to strings.  `None` fields
+        are omitted.
+        """
+        return json.loads(self.json(exclude_none=True, cls=HandleKeyEnumEncoder))
+
+    @validator("schemaKey", always=True)
+    def ensure_schemakey(cls, val):
+        tempval = val
+        if "Published" in cls.__name__:
+            tempval = "Published" + tempval
+        elif "BareAsset" == cls.__name__:
+            tempval = "Bare" + tempval
+        if tempval != cls.__name__:
+            raise ValueError(
+                f"schemaKey {tempval} does not match classname {cls.__name__}"
+            )
+        return val
+
     @classmethod
     def unvalidated(__pydantic_cls__: Type[BaseModel], **data: Any) -> BaseModel:
         """Allow model to be returned without validation"""
@@ -132,7 +179,7 @@ class DandiBaseModel(BaseModel, metaclass=DandiBaseModelMetaclass):
         return self
 
     @classmethod
-    def to_dictrepr(__pydantic_cls__: Type[BaseModel]):
+    def to_dictrepr(__pydantic_cls__: Type["DandiBaseModel"]):
         return (
             __pydantic_cls__.unvalidated()
             .__repr__()
@@ -152,6 +199,8 @@ class DandiBaseModel(BaseModel, metaclass=DandiBaseModelMetaclass):
                         del value["pattern"]
                 if value.get("title") is None or value["title"] == prop.title():
                     value["title"] = name2title(prop)
+                if re.match("\\^https?://", value.get("pattern", "")):
+                    value["format"] = "uri"
                 if value.get("format", None) == "uri":
                     value["maxLength"] = 1000
                 allOf = value.get("allOf")
@@ -186,20 +235,25 @@ class DandiBaseModel(BaseModel, metaclass=DandiBaseModelMetaclass):
                         del value["enum"]
                     else:
                         value["const"] = value["default"]
+                    if "readOnly" in value:
+                        del value["readOnly"]
 
 
 class PropertyValue(DandiBaseModel):
-    maxValue: float = Field(None, nskey="schema")
-    minValue: float = Field(None, nskey="schema")
-    unitText: str = Field(None, nskey="schema")
-    value: Union[Any, List[Any]] = Field(None, nskey="schema")
-    valueReference: "PropertyValue" = Field(
+    maxValue: Optional[float] = Field(None, nskey="schema")
+    minValue: Optional[float] = Field(None, nskey="schema")
+    unitText: Optional[str] = Field(None, nskey="schema")
+    value: Union[Any, List[Any]] = Field(
+        nskey="schema", description="The value associated with this property."
+    )
+    valueReference: Optional["PropertyValue"] = Field(
         None, nskey="schema"
     )  # Note: recursive (circular or not)
-    propertyID: Union[IdentifierType, HttpUrl] = Field(
+    propertyID: Optional[Union[IdentifierType, HttpUrl]] = Field(
         None,
         description="A commonly used identifier for"
-        "the characteristic represented by the property.",
+        "the characteristic represented by the property. "
+        "For example, a known prefix like DOI or a full URL.",
         nskey="schema",
     )
 
@@ -213,22 +267,47 @@ ORCID = str
 RORID = str
 DANDI = str
 RRID = str
+DANDIURL = str
 
 
 class BaseType(DandiBaseModel):
     """Base class for enumerated types"""
 
     identifier: Optional[Union[HttpUrl, str]] = Field(
+        None,
         description="The identifier can be any url or a compact URI, preferably"
-        " supported by identifiers.org",
-        regex=r"^[a-zA-Z0-9]+:[a-zA-Z0-9-/\.]+$",
+        " supported by identifiers.org.",
+        regex=r"^[a-zA-Z0-9]+:[a-zA-Z0-9-/\._]+$",
         nskey="schema",
     )
     name: Optional[str] = Field(
-        description="The name of the item.", max_length=150, nskey="schema"
+        None, description="The name of the item.", max_length=150, nskey="schema"
     )
     _ldmeta = {"rdfs:subClassOf": ["prov:Entity", "schema:Thing"], "nskey": "dandi"}
 
+    class Config:
+        @staticmethod
+        def schema_extra(schema: Dict[str, Any], model: Type["BaseType"]) -> None:
+            schema["title"] = name2title(schema["title"])
+            for prop, value in schema.get("properties", {}).items():
+                # This check removes the anyOf field from the identifier property
+                # in the schema generation. This relates to a UI issue where two
+                # basic properties, in this case "string", is dropped from the UI.
+                if prop == "identifier":
+                    for option in value.pop("anyOf", []):
+                        if option.get("format", "") == "uri":
+                            value.update(**option)
+                            value["maxLength"] = 1000
+                # Have to replicate this
+                if prop == "schemaKey":
+                    if "enum" in value and len(value["enum"]) == 1:
+                        value["const"] = value["enum"][0]
+                        del value["enum"]
+                    else:
+                        value["const"] = value["default"]
+                    if "readOnly" in value:
+                        del value["readOnly"]
+
 
 class AssayType(BaseType):
     """OBI based identifier for the assay(s) used"""
@@ -282,9 +361,25 @@ class StandardsType(BaseType):
     """Identifier for data standard used"""
 
 
+nwb_standard = StandardsType(
+    name="Neurodata Without Borders (NWB)", identifier="RRID:SCR_015242"
+).json_dict()
+
+
+bids_standard = StandardsType(
+    name="Brain Imaging Data Structure (BIDS)", identifier="RRID:SCR_016124"
+).json_dict()
+
+
 class ContactPoint(DandiBaseModel):
-    email: Optional[EmailStr] = Field(None, nskey="schema")
-    url: Optional[HttpUrl] = Field(None, nskey="schema")
+    email: Optional[EmailStr] = Field(
+        None, description="Email address of contact.", nskey="schema"
+    )
+    url: Optional[HttpUrl] = Field(
+        None,
+        description="A Web page to find information on how to contact.",
+        nskey="schema",
+    )
 
     _ldmeta = {"nskey": "schema"}
 
@@ -293,26 +388,30 @@ class Contributor(DandiBaseModel):
     identifier: Optional[Identifier] = Field(
         None,
         title="A common identifier",
-        description="Use a common identifier such as ORCID for people or ROR for institutions",
+        description="Use a common identifier such as ORCID (orcid.org) for "
+        "people or ROR (ror.org) for institutions.",
         nskey="schema",
     )
     name: Optional[str] = Field(None, nskey="schema")
     email: Optional[EmailStr] = Field(None, nskey="schema")
     url: Optional[HttpUrl] = Field(None, nskey="schema")
     roleName: Optional[List[RoleType]] = Field(
-        None, title="Role", description="Role of the contributor", nskey="schema"
+        None,
+        title="Role",
+        description="Role(s) of the contributor. Multiple roles can be selected.",
+        nskey="schema",
     )
     includeInCitation: bool = Field(
         True,
         title="Include contributor in citation",
         description="A flag to indicate whether a contributor should be included "
-        "when generating a citation for the item",
+        "when generating a citation for the item.",
         nskey="dandi",
     )
     awardNumber: Optional[Identifier] = Field(
         None,
         title="Identifier for an award",
-        description="Identifier associated with a sponsored or gift award",
+        description="Identifier associated with a sponsored or gift award.",
         nskey="dandi",
     )
 
@@ -321,7 +420,7 @@ class Organization(Contributor):
     identifier: Optional[RORID] = Field(
         None,
         title="A ror.org identifier",
-        description="Use an ror.org identifier for institutions",
+        description="Use an ror.org identifier for institutions.",
         regex=r"^https://ror.org/[a-z0-9]+$",
         nskey="schema",
     )
@@ -349,11 +448,11 @@ class Affiliation(DandiBaseModel):
     identifier: Optional[RORID] = Field(
         None,
         title="A ror.org identifier",
-        description="Use an ror.org identifier for institutions",
+        description="Use an ror.org identifier for institutions.",
         regex=r"^https://ror.org/[a-z0-9]+$",
         nskey="schema",
     )
-    name: str = Field(None, nskey="schema")
+    name: str = Field(nskey="schema", description="Name of organization")
 
     _ldmeta = {
         "rdfs:subClassOf": ["schema:Organization", "prov:Organization"],
@@ -365,7 +464,7 @@ class Person(Contributor):
     identifier: Optional[ORCID] = Field(
         None,
         title="An ORCID identifier",
-        description="An ORCID (orcid.org) identifier for an individual",
+        description="An ORCID (orcid.org) identifier for an individual.",
         regex=r"^\d{4}-\d{4}-\d{4}-(\d{3}X|\d{4})$",
         nskey="schema",
     )
@@ -375,7 +474,7 @@ class Person(Contributor):
         nskey="schema",
         examples=["Lovelace, Augusta Ada", "Smith, John", "Chan, Kong-sang"],
     )
-    affiliation: List[Affiliation] = Field(
+    affiliation: Optional[List[Affiliation]] = Field(
         None,
         description="An organization that this person is affiliated with.",
         nskey="schema",
@@ -395,7 +494,7 @@ class Software(DandiBaseModel):
     name: str = Field(nskey="schema")
     version: str = Field(nskey="schema")
     url: Optional[HttpUrl] = Field(
-        None, description="Web page for the software", nskey="schema"
+        None, description="Web page for the software.", nskey="schema"
     )
 
     _ldmeta = {
@@ -408,7 +507,7 @@ class Agent(DandiBaseModel):
     identifier: Optional[Identifier] = Field(
         None,
         title="Identifier",
-        description="Identifier for an agent",
+        description="Identifier for an agent.",
         nskey="schema",
     )
     name: str = Field(nskey="schema")
@@ -428,8 +527,10 @@ class EthicsApproval(DandiBaseModel):
         title="Approved protocol identifier",
         description="Approved Protocol identifier, often a number or alphanumeric string.",
     )
-    contactPoint: ContactPoint = Field(
-        description="Information about the ethics approval committee.", nskey="schema"
+    contactPoint: Optional[ContactPoint] = Field(
+        None,
+        description="Information about the ethics approval committee.",
+        nskey="schema",
     )
 
     _ldmeta = {"rdfs:subClassOf": ["schema:Thing", "prov:Entity"], "nskey": "dandi"}
@@ -438,17 +539,17 @@ class EthicsApproval(DandiBaseModel):
 class Resource(DandiBaseModel):
     identifier: Optional[Identifier] = Field(None, nskey="schema")
     name: Optional[str] = Field(None, title="A title of the resource", nskey="schema")
-    url: HttpUrl = Field(None, title="URL of the resource", nskey="schema")
+    url: Optional[HttpUrl] = Field(None, title="URL of the resource", nskey="schema")
     repository: Optional[str] = Field(
         None,
         title="Name of the repository",
-        description="Name of the repository in which the resource is housed",
+        description="Name of the repository in which the resource is housed.",
         nskey="dandi",
     )
     relation: RelationType = Field(
         title="Resource relation",
         description="Indicates how the resource is related to the dataset. "
-        "This relation should satisfy: dandiset <relation> resource",
+        "This relation should satisfy: dandiset <relation> resource.",
         nskey="dandi",
     )
 
@@ -459,18 +560,25 @@ class Resource(DandiBaseModel):
         "nskey": "dandi",
     }
 
+    @root_validator
+    def identifier_or_url(cls, values):
+        identifier, url = values.get("identifier"), values.get("url")
+        if identifier is None and url is None:
+            raise ValueError("Both identifier and url cannot be None")
+        return values
+
 
 class AccessRequirements(DandiBaseModel):
     """Information about access options for the dataset"""
 
     status: AccessType = Field(
         title="Access status",
-        description="The access status of the item",
+        description="The access status of the item.",
         nskey="dandi",
     )
     contactPoint: Optional[ContactPoint] = Field(
         None,
-        description="Who or where to look for information about access",
+        description="Who or where to look for information about access.",
         nskey="schema",
     )
     description: Optional[str] = Field(
@@ -481,7 +589,7 @@ class AccessRequirements(DandiBaseModel):
     embargoedUntil: Optional[date] = Field(
         None,
         title="Embargo end date",
-        description="Date on which embargo ends",
+        description="Date on which embargo ends.",
         readOnly=True,
         nskey="dandi",
         rangeIncludes="schema:Date",
@@ -489,24 +597,32 @@ class AccessRequirements(DandiBaseModel):
 
     _ldmeta = {"rdfs:subClassOf": ["schema:Thing", "prov:Entity"], "nskey": "dandi"}
 
+    """
+    # TODO: Enable when embargoed access is supported.
+    @root_validator
+    def open_or_embargoed(cls, values):
+        status, embargoed = values.get("status"), values.get("embargoedUntil")
+        if status == AccessType.EmbargoedAccess and embargoed is None:
+            raise ValueError("An embargo date. For NIH supported awards, this "
+            "must comply with NIH policy. For other datasets, this will be at "
+            "most a year from the dandiset creation.")
+        return values
+    """
+
 
 class AssetsSummary(DandiBaseModel):
     """Summary over assets contained in a dandiset (published or not)"""
 
     # stats which are not stats
-    numberOfBytes: int = Field(readOnly=True, gt=0, sameas="schema:contentSize")
-    numberOfFiles: int = Field(readOnly=True, gt=0)  # universe
+    numberOfBytes: int = Field(readOnly=True, sameas="schema:contentSize")
+    numberOfFiles: int = Field(readOnly=True)  # universe
     numberOfSubjects: Optional[int] = Field(None, readOnly=True)  # NWB + BIDS
     numberOfSamples: Optional[int] = Field(None, readOnly=True)  # more of NWB
     numberOfCells: Optional[int] = Field(None, readOnly=True)
 
-    dataStandard: Optional[List[StandardsType]] = Field(
-        readOnly=True
-    )  # TODO: types of things NWB, BIDS
+    dataStandard: Optional[List[StandardsType]] = Field(readOnly=True)
     # Web UI: icons per each modality?
-    approach: Optional[List[ApproachType]] = Field(
-        readOnly=True
-    )  # TODO: types of things, BIDS etc...
+    approach: Optional[List[ApproachType]] = Field(readOnly=True)
     # Web UI: could be an icon with number, which if hovered on  show a list?
     measurementTechnique: Optional[List[MeasurementTechniqueType]] = Field(
         readOnly=True, nskey="schema"
@@ -531,7 +647,7 @@ class Equipment(DandiBaseModel):
         nskey="schema",
     )
     description: Optional[str] = Field(
-        None, description="The description of the activity.", nskey="schema"
+        None, description="The description of the equipment.", nskey="schema"
     )
 
     _ldmeta = {
@@ -561,7 +677,9 @@ class Activity(DandiBaseModel):
     wasAssociatedWith: Optional[
         List[Union[Person, Organization, Software, Agent]]
     ] = Field(None, nskey="prov")
-    used: Optional[List[Equipment]] = Field(None, nskey="prov")
+    used: Optional[List[Equipment]] = Field(
+        None, description="A listing of equipment used for the activity.", nskey="prov"
+    )
 
     _ldmeta = {"rdfs:subClassOf": ["prov:Activity", "schema:Thing"], "nskey": "dandi"}
 
@@ -596,7 +714,7 @@ class PublishActivity(Activity):
 
 class Locus(DandiBaseModel):
     identifier: Union[Identifier, List[Identifier]] = Field(
-        description="Identifier for genotyping locus", nskey="schema"
+        description="Identifier for genotyping locus.", nskey="schema"
     )
     locusType: Optional[str] = Field(None)
     _ldmeta = {"nskey": "dandi"}
@@ -604,7 +722,7 @@ class Locus(DandiBaseModel):
 
 class Allele(DandiBaseModel):
     identifier: Union[Identifier, List[Identifier]] = Field(
-        description="Identifier for genotyping allele", nskey="schema"
+        description="Identifier for genotyping allele.", nskey="schema"
     )
     alleleSymbol: Optional[str] = Field(None)
     alleleType: Optional[str] = Field(None)
@@ -612,44 +730,60 @@ class Allele(DandiBaseModel):
 
 
 class GenotypeInfo(DandiBaseModel):
-    locus: Locus = Field(description="Locus at which information was extracted")
-    alleles: List[Allele] = Field(description="Information about alleles at the locus")
-    wasGeneratedBy: Optional[List["Session"]] = Field(None, nskey="prov")
+    locus: Locus = Field(description="Locus at which information was extracted.")
+    alleles: List[Allele] = Field(description="Information about alleles at the locus.")
+    wasGeneratedBy: Optional[List["Session"]] = Field(
+        None,
+        description="Information about session activity used to determine genotype.",
+        nskey="prov",
+    )
     _ldmeta = {"nskey": "dandi"}
 
 
 class RelatedParticipant(DandiBaseModel):
     identifier: Optional[Identifier] = Field(None, nskey="schema")
-    name: Optional[str] = Field(None, title="Name of the participant", nskey="schema")
+    name: Optional[str] = Field(
+        None, title="Name of the participant or subject", nskey="schema"
+    )
     url: Optional[HttpUrl] = Field(
-        None, title="URL of the related participant", nskey="schema"
+        None, title="URL of the related participant or subject", nskey="schema"
     )
     relation: ParticipantRelationType = Field(
-        title="Participant relation",
-        description="Indicates how the current participant is related to the other participant "
-        "This relation should satisfy: Participant <relation> relatedParticipant",
+        title="Participant or subject relation",
+        description="Indicates how the current participant or subject is related "
+        "to the other participant or subject. This relation should "
+        "satisfy: Participant/Subject <relation> relatedParticipant/Subject.",
         nskey="dandi",
     )
 
     _ldmeta = {
         "rdfs:subClassOf": ["schema:CreativeWork", "prov:Entity"],
-        "rdfs:comment": "Another participant related to the participant (e.g., another "
-        "parent, sibling, child)",
+        "rdfs:comment": "Another participant or subject related to the current "
+        "participant or subject (e.g., another parent, sibling, child).",
         "nskey": "dandi",
     }
 
 
 class Participant(DandiBaseModel):
-    """Description about the sample that was studied"""
+    """Description about the Participant or Subject studied.
 
-    identifier: Optional[Identifier] = Field(nskey="schema")
+    The Participant or Subject can be any individual or synthesized Agent. The
+    properties of the Participant or Subject refers to information at the timepoint
+    when the Participant or Subject engaged in the production of data being described.
+    """
+
+    identifier: Identifier = Field(nskey="schema")
     altName: Optional[List[Identifier]] = Field(None, nskey="dandi")
 
     strain: Optional[StrainType] = Field(
-        None, description="Identifier for the strain of the sample", nskey="dandi"
+        None,
+        description="Identifier for the strain of the participant or subject.",
+        nskey="dandi",
     )
     cellLine: Optional[Identifier] = Field(
-        None, description="Cell line associated with the sample", nskey="dandi"
+        None,
+        description="Cell line associated with the participant or subject.",
+        nskey="dandi",
     )
     vendor: Optional[Organization] = Field(None, nskey="dandi")
     age: Optional[PropertyValue] = Field(
@@ -663,29 +797,43 @@ class Participant(DandiBaseModel):
 
     sex: Optional[SexType] = Field(
         None,
-        description="OBI based identifier for sex of the sample if available",
+        description="Identifier for sex of the participant or subject if "
+        "available. (e.g. from OBI)",
         nskey="dandi",
     )
     genotype: Optional[Union[List[GenotypeInfo], Identifier]] = Field(
-        None, description="Genotype descriptor of biosample if available", nskey="dandi"
+        None,
+        description="Genotype descriptor of participant or subject if available",
+        nskey="dandi",
     )
     species: Optional[SpeciesType] = Field(
         None,
-        description="An identifier indicating the taxonomic classification of the biosample",
+        description="An identifier indicating the taxonomic classification of "
+        "the participant or subject.",
         nskey="dandi",
     )
     disorder: Optional[List[Disorder]] = Field(
         None,
-        description="Any current diagnosed disease or disorder associated with the sample",
+        description="Any current diagnosed disease or disorder associated with "
+        "the participant or subject.",
         nskey="dandi",
     )
 
-    relatedParticipant: Optional[List[RelatedParticipant]] = Field(None, nskey="dandi")
-    sameAs: Optional[List[Identifier]] = Field(None, nskey="schema")
+    relatedParticipant: Optional[List[RelatedParticipant]] = Field(
+        None,
+        description="Information about related participants or subjects in a "
+        "study or across studies.",
+        nskey="dandi",
+    )
+    sameAs: Optional[List[Identifier]] = Field(
+        None,
+        description="An identifier to link participants or subjects across datasets.",
+        nskey="schema",
+    )
 
     _ldmeta = {
         "rdfs:subClassOf": ["prov:Agent"],
-        "rdfs:label": "Information about the participant.",
+        "rdfs:label": "Information about the participant or subject.",
         "nskey": "dandi",
     }
 
@@ -693,23 +841,30 @@ class Participant(DandiBaseModel):
 class BioSample(DandiBaseModel):
     """Description of the sample that was studied"""
 
-    identifier: Optional[Identifier] = Field(nskey="schema")
-    sampleType: Optional[SampleType] = Field(
-        None, description="OBI based identifier for the sample used", nskey="dandi"
+    identifier: Identifier = Field(nskey="schema")
+    sampleType: SampleType = Field(
+        description="Identifier for the sample characteristics (e.g., from OBI, Encode).",
+        nskey="dandi",
     )
     assayType: Optional[List[AssayType]] = Field(
-        None, description="OBI based identifier for the assay(s) used", nskey="dandi"
+        None, description="Identifier for the assay(s) used (e.g., OBI).", nskey="dandi"
     )
     anatomy: Optional[List[Anatomy]] = Field(
         None,
-        description="UBERON based identifier for what organ the sample belongs "
-        "to. Use the most specific descriptor.",
+        description="Identifier for what organ the sample belongs "
+        "to. Use the most specific descriptor from sources such as UBERON.",
         nskey="dandi",
     )
 
-    wasDerivedFrom: Optional[List["BioSample"]] = Field(None, nskey="prov")
+    wasDerivedFrom: Optional[List["BioSample"]] = Field(
+        None,
+        description="Describes the hierarchy of sample derivation or aggregation.",
+        nskey="prov",
+    )
     wasAttributedTo: Optional[List[Participant]] = Field(
-        None, description="Participant(s) to which this sample belongs to", nskey="prov"
+        None,
+        description="Participant(s) or Subject(s) associated with this sample.",
+        nskey="prov",
     )
     sameAs: Optional[List[Identifier]] = Field(None, nskey="schema")
     hasMember: Optional[List[Identifier]] = Field(None, nskey="prov")
@@ -741,7 +896,7 @@ class CommonModel(DandiBaseModel):
     contributor: Optional[List[Union[Person, Organization]]] = Field(
         None,
         title="Contributors",
-        description="Contributors to this item.",
+        description="Contributors to this item: persons or organizations.",
         nskey="schema",
     )
     about: Optional[List[Union[Disorder, Anatomy, GenericType]]] = Field(
@@ -751,26 +906,36 @@ class CommonModel(DandiBaseModel):
         nskey="schema",
     )
     studyTarget: Optional[List[str]] = Field(
-        None, description="What the study is related to", nskey="dandi"
+        None,
+        description="Objectives or specific questions of the study.",
+        nskey="dandi",
     )
     license: Optional[List[LicenseType]] = Field(
-        None, description="Licenses associated with the item.", nskey="schema"
+        None,
+        description="Licenses associated with the item. DANDI only supports a "
+        "subset of Creative Commons Licenses (creativecommons.org) "
+        "applicable to datasets.",
+        nskey="schema",
     )
     protocol: Optional[List[HttpUrl]] = Field(
-        None, description="A list of protocol.io URLs", nskey="dandi"
+        None,
+        description="A list of persistent URLs describing the protocol (e.g. "
+        "protocols.io, or other DOIs).",
+        nskey="dandi",
     )
     ethicsApproval: Optional[List[EthicsApproval]] = Field(
         None, title="Ethics approvals", nskey="dandi"
     )
     keywords: Optional[List[str]] = Field(
         None,
-        description="Keywords or tags used to describe "
-        "this content. Multiple entries in a "
-        "keywords list are typically delimited "
-        "by commas.",
+        description="Keywords used to describe this content.",
         nskey="schema",
     )
-    acknowledgement: Optional[str] = Field(None, nskey="dandi")
+    acknowledgement: Optional[str] = Field(
+        None,
+        descriptions="Any acknowledgments not covered by contributors or external resources.",
+        nskey="dandi",
+    )
 
     # Linking to this dandiset or the larger thing
     access: List[AccessRequirements] = Field(
@@ -791,14 +956,6 @@ class CommonModel(DandiBaseModel):
 
     wasGeneratedBy: Optional[List[Activity]] = Field(None, nskey="prov")
 
-    def json_dict(self):
-        """
-        Recursively convert the instance to a `dict` of JSONable values,
-        including converting enum values to strings.  `None` fields
-        are omitted.
-        """
-        return json.loads(self.json(exclude_none=True, cls=HandleKeyEnumEncoder))
-
 
 class Dandiset(CommonModel):
     """A body of structured information describing a DANDI dataset."""
@@ -822,7 +979,7 @@ class Dandiset(CommonModel):
     identifier: DANDI = Field(
         readOnly=True,
         title="Dandiset identifier",
-        description="A Dandiset identifier that can be resolved by identifiers.org",
+        description="A Dandiset identifier that can be resolved by identifiers.org.",
         regex=r"^DANDI\:\d{6}$",
         nskey="schema",
     )
@@ -842,14 +999,18 @@ class Dandiset(CommonModel):
         min_items=1,
     )
     dateCreated: Optional[datetime] = Field(
-        nskey="schema", title="Dandiset creation date and time", readOnly=True
+        None, nskey="schema", title="Dandiset creation date and time.", readOnly=True
     )
     dateModified: Optional[datetime] = Field(
-        nskey="schema", title="Last modification date and time", readOnly=True
+        None, nskey="schema", title="Last modification date and time.", readOnly=True
     )
 
     license: List[LicenseType] = Field(
-        min_items=1, description="Licenses associated with the item.", nskey="schema"
+        min_items=1,
+        description="Licenses associated with the item. DANDI only supports a "
+        "subset of Creative Commons Licenses (creativecommons.org) "
+        "applicable to datasets.",
+        nskey="schema",
     )
 
     citation: str = Field(readOnly=True, nskey="schema")
@@ -865,7 +1026,7 @@ class Dandiset(CommonModel):
     wasGeneratedBy: Optional[List[Project]] = Field(
         None,
         title="Associated projects",
-        description="Describe the project(s) that generated this Dandiset",
+        description="Project(s) that generated this Dandiset.",
         nskey="prov",
     )
 
@@ -892,10 +1053,19 @@ class BareAsset(CommonModel):
     path: str = Field(nskey="dandi")
 
     dateModified: Optional[datetime] = Field(
-        nskey="schema", title="Asset (file or metadata) modification date and time"
+        None,
+        nskey="schema",
+        title="Asset (file or metadata) modification date and time",
     )
     blobDateModified: Optional[datetime] = Field(
-        nskey="dandi", title="Asset file modification date and time"
+        None, nskey="dandi", title="Asset file modification date and time."
+    )
+    # overload to restrict with max_items=1
+    access: List[AccessRequirements] = Field(
+        title="Access information",
+        default_factory=lambda: [AccessRequirements(status=AccessType.OpenAccess)],
+        nskey="dandi",
+        max_items=1,
     )
 
     # this is from C2M2 level 1 - using EDAM vocabularies - in our case we would
@@ -915,13 +1085,15 @@ class BareAsset(CommonModel):
     )
 
     wasDerivedFrom: Optional[List[BioSample]] = Field(None, nskey="prov")
-    wasAttributedTo: List[Participant] = Field(
-        None, description="Participant(s) to which this file belongs to", nskey="prov"
+    wasAttributedTo: Optional[List[Participant]] = Field(
+        None,
+        description="Associated participant(s) or subject(s).",
+        nskey="prov",
     )
     wasGeneratedBy: Optional[List[Union[Session, Project, Activity]]] = Field(
         None,
         title="Name of the session, project or activity.",
-        description="Describe the session, project or activity that generated this asset",
+        description="Describe the session, project or activity that generated this asset.",
         nskey="prov",
     )
 
@@ -954,9 +1126,9 @@ class Asset(BareAsset):
     """Metadata used to describe an asset on the server."""
 
     # all of the following are set by server
-    id: str = Field(readOnly=True, description="Uniform resource identifier")
+    id: str = Field(readOnly=True, description="Uniform resource identifier.")
     identifier: UUID4 = Field(readOnly=True, nskey="schema")
-    contentUrl: List[HttpUrl] = Field(None, readOnly=True, nskey="schema")
+    contentUrl: List[HttpUrl] = Field(readOnly=True, nskey="schema")
 
 
 class Publishable(DandiBaseModel):
@@ -964,26 +1136,49 @@ class Publishable(DandiBaseModel):
         description="The URL should contain the provenance of the publishing process.",
         readOnly=True,
         nskey="dandi",
-    )  # TODO: formalize "publish" activity to at least the Actor
+    )
     datePublished: datetime = Field(readOnly=True, nskey="schema")
 
 
 class PublishedDandiset(Dandiset, Publishable):
+    id: str = Field(
+        description="Uniform resource identifier.",
+        regex=DANDI_PUBID_PATTERN,
+        readOnly=True,
+    )
+
     doi: str = Field(
         title="DOI",
         readOnly=True,
-        regex=r"^10\.[A-Za-z0-9.\/-]+",
+        regex=DANDI_DOI_PATTERN,
         nskey="dandi",
     )
-    url: HttpUrl = Field(
-        readOnly=True, description="permalink to the item", nskey="schema"
+    url: DANDIURL = Field(
+        readOnly=True,
+        description="Permalink to the Dandiset.",
+        regex=r"^https://dandiarchive.org/dandiset/\d{6}/\d+\.\d+\.\d+$",
+        nskey="schema",
     )
 
     schemaKey = "Dandiset"
 
+    @validator("assetsSummary")
+    def check_filesbytes(cls, values):
+        if values.numberOfBytes == 0 or values.numberOfFiles == 0:
+            raise ValueError(
+                "A Dandiset containing no files or zero bytes is not publishable"
+            )
+        return values
+
 
 class PublishedAsset(Asset, Publishable):
 
+    id: str = Field(
+        description="Uniform resource identifier.",
+        regex=ASSET_UUID_PATTERN,
+        readOnly=True,
+    )
+
     schemaKey = "Asset"
 
     @validator("digest")
